from .base_lib import BaseLib
from .puzzle import Puzzle
from .kissmanga_com import KissMangaComCrypt
from .manga_rock_com import MangaRockComCrypt
from .manhuagui_com import ManhuaGuiComCrypt
from .sunday_webry_com import SundayWebryCom
from .ac_qq_com import AcQqComCrypt



from manga_py.provider import Provider
from .helpers.std import Std


class ComicExtra(Provider, Std):

    def get_archive_name(self) -> str:
        return 'vol_{:0>3}'.format(self._storage['current_chapter'])

    def get_chapter_index(self) -> str:
        return self._storage['current_chapter']
        # return self.re.search('(\d+)', self.chapter).group(1)

    def get_main_content(self):
        return self._get_content('{}/comic/{}')

    def get_manga_name(self):
        url = self.get_url()
        test = self.re.search('/comic/([^/]+)', url)
        if test:
            return test.group(1)
        return self.re.search('/([^/]+)/chapter', url).group(1)

    def get_chapters(self):
        return self.document_fromstring(self.content, '#list td a')

    def get_files(self):
        url = self.chapter + '/full'
        items = self.html_fromstring(url, '.chapter-container img.chapter_img')
        return [i.get('src') for i in items]

    def get_cover(self):
        return self._cover_from_content('.movie-image img')


main = ComicExtra



from manga_py.crypt import AcQqComCrypt
from manga_py.provider import Provider
from .helpers.std import Std


class AcQqCom(Provider, Std):
    _decoder = None

    def get_archive_name(self) -> str:
        return 'vol_{:0>3}'.format(self.get_chapter_index())

    def get_chapter_index(self) -> str:
        return self.re.search(r'/cid/(\d+)', self.chapter).group(1)

    def get_main_content(self):
        content = self._storage.get('main_content', None)
        if content is not None:
            return content
        idx = self._get_name(r'/id/(\d+)')
        return self.http_get('{}/Comic/comicInfo/id/{}'.format(self.domain, idx))

    def get_manga_name(self) -> str:
        title = self.document_fromstring(self.content, '.works-intro-title strong', 0)
        return title.text_content()

    def get_chapters(self):
        return self._elements('.chapter-page-all li a')[::-1]

    def get_files(self):
        data = self._decoder.decode()
        return [i.get('url') for i in data.get('picture', [])]

    def get_cover(self) -> str:
        return self._cover_from_content('.works-cover img')

    def prepare_cookies(self):
        self._decoder = AcQqComCrypt(self)
        self._base_cookies()


main = AcQqCom



from manga_py.provider import Provider
from .helpers.std import Std


class MangaHereCc(Provider, Std):

    def get_archive_name(self) -> str:
        return 'vol_{:0>3}'.format(self.get_chapter_index())

    def get_chapter_index(self) -> str:
        selector = r'/manga/[^/]+/[^\d]+(\d+)'
        chapter = self.chapter
        return self.re.search(selector, chapter).group(1)

    def get_main_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name('/manga/([^/]+)')

    def get_chapters(self):
        return self._elements('.detail_list .left a')

    @staticmethod
    def __get_img(parser):
        return parser.cssselect('img#image')[0].get('src')

    def get_files(self):
        parser = self.html_fromstring(self.chapter)
        pages = parser.cssselect('.go_page select.wid60 option + option')
        pages_list = [value.get('value') for value in pages]
        first_image = self.__get_img(parser)
        images = [first_image]
        for i in pages_list:
            parser = self.html_fromstring(i)
            images.append(self.__get_img(parser))
        return images

    def get_cover(self):
        return self._cover_from_content('.manga_detail_top > img')

    def prepare_cookies(self):
        self._base_cookies()


main = MangaHereCc



try:
    from atexit import register as atexit_register
    from os import makedirs
    from os import path
    from shutil import rmtree
    from sys import argv, exit

    from manga_py.cli import Cli, get_cli_arguments
    from manga_py.fs import get_temp_path

    __author__ = 'Sergey Zharkov'
    __license__ = 'MIT'
    __email__ = 'sttv-pc@mail.ru'


    @atexit_register
    def before_shutdown():
        temp_dir = get_temp_path()
        path.isdir(temp_dir) and rmtree(temp_dir)


    def main():
        temp_path = get_temp_path()
        path.isdir(temp_path) or makedirs(temp_path)

        args = get_cli_arguments()
        parse_args = args.parse_args()

        # if parse_args.server:
        #     server_mode = Server(args)
        #     exit(1 if server_mode.status else 0)

        cli_mode = Cli(args)
        cli_mode.start()
        exit(1 if cli_mode.status else 0)

except Exception:
    def main():
        print('Setup progress?')



from manga_py.provider import Provider
from .helpers.std import Std


class SoMangaNet(Provider, Std):

    def get_archive_name(self) -> str:
        return 'vol_{:0>3}'.format(self.get_chapter_index())

    def get_chapter_index(self) -> str:
        return self.re.search('/leitor/[^/]+/([^/]+)').group(1)

    def get_main_content(self):
        return self._get_content('{}/manga/{}')

    def get_manga_name(self) -> str:
        return self._get_name(r'\.net/[^/]+/([^/]+)')

    def get_chapters(self):
        return self.document_fromstring(self.content, 'ul.capitulos li > a')

    def get_files(self):
        parser = self.html_fromstring(self.chapter, 'img.img-manga')
        return [i.get('src') for i in parser]

    def get_cover(self):
        pass  # FIXME HOME


main = SoMangaNet



from zipfile import ZipFile, ZIP_DEFLATED
from PIL import Image

from manga_py.fs import is_file, make_dirs, basename, dirname, unlink
from os.path import splitext


class Archive:
    files = None

    def __init__(self):
        self.files = []

    @staticmethod
    def _check_ext(file, name):
        file_name, file_ext = splitext(name)
        if file_ext == '' or file_ext == '.':
            try:
                image = Image.open(file)
                name = file_name.rstrip('.') + '.' + image.format.lower()
                image.close()
            except (FileNotFoundError, OSError):
                pass
        return name

    def add_file(self, file, in_arc_name=None):
        if in_arc_name is None:
            in_arc_name = basename(file)
        in_arc_name = self._check_ext(file, in_arc_name)
        self.files.append((file, in_arc_name))

    def set_files_list(self, files):
        self.files = files

    def make(self, dst, info_file=None):
        if not len(self.files):
            return

        make_dirs(dirname(dst))
        archive = ZipFile(dst, 'w', ZIP_DEFLATED)

        for file in self.files:
            if is_file(file[0]):
                archive.write(file[0], file[1])

        info_file and archive.writestr('info.txt', info_file)

        archive.close()

        self._maked()

    def _maked(self):
        for file in self.files:
            unlink(file[0])
